@@ -1,6 +1,6 @@
 ;; c99-act.scm
 
-;; Copyright (C) 2016-2018 Matthew R. Wette
+;; Copyright (C) 2016-2020 Matthew R. Wette
 ;; 
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -61,11 +61,9 @@
    ;; argument-expression-list => argument-expression-list "," arg-expr-hack
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; arg-expr-hack => declaration-specifiers abstract-declarator
-   (lambda ($2 $1 . $rest)
-     `(param-decl ,(tl->list $1) $2))
+   (lambda ($2 $1 . $rest) `(param-decl ,1 ,$2))
    ;; arg-expr-hack => declaration-specifiers
-   (lambda ($1 . $rest)
-     `(param-decl ,(tl->list $1)))
+   (lambda ($1 . $rest) `(param-decl ,$1))
    ;; unary-expression => postfix-expression
    (lambda ($1 . $rest) $1)
    ;; unary-expression => "++" unary-expression
@@ -362,8 +360,8 @@
    ;; fixpt-type-specifier => "signed" "_Fract"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "signed _Fract"))
-   ;; fixpt-type-specifier => "signed" "long _Fract"
-   (lambda ($2 $1 . $rest)
+   ;; fixpt-type-specifier => "signed" "long" "_Fract"
+   (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "signed long _Fract"))
    ;; fixpt-type-specifier => "unsigned" "short" "_Fract"
    (lambda ($3 $2 $1 . $rest)
@@ -371,16 +369,17 @@
    ;; fixpt-type-specifier => "unsigned" "_Fract"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "unsigned _Fract"))
-   ;; fixpt-type-specifier => "unsigned" "long _Fract"
-   (lambda ($2 $1 . $rest)
+   ;; fixpt-type-specifier => "unsigned" "long" "_Fract"
+   (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "unsigned long _Fract"))
    ;; fixpt-type-specifier => "short" "_Accum"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "short _Accum"))
    ;; fixpt-type-specifier => "_Accum"
    (lambda ($1 . $rest) '(fixpt-type "_Accum"))
-   ;; fixpt-type-specifier => "long _Accum"
-   (lambda ($1 . $rest) '(fixpt-type "long _Accum"))
+   ;; fixpt-type-specifier => "long" "_Accum"
+   (lambda ($2 $1 . $rest)
+     '(fixpt-type "long _Accum"))
    ;; fixpt-type-specifier => "signed" "short" "_Accum"
    (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "signd short _Accum"))
@@ -414,8 +413,8 @@
    ;; fixpt-type-specifier => "_Sat" "signed" "_Fract"
    (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "_Sat signed _Fract"))
-   ;; fixpt-type-specifier => "_Sat" "signed" "long _Fract"
-   (lambda ($3 $2 $1 . $rest)
+   ;; fixpt-type-specifier => "_Sat" "signed" "long" "_Fract"
+   (lambda ($4 $3 $2 $1 . $rest)
      '(fixpt-type "_Sat signed long _Fract"))
    ;; fixpt-type-specifier => "_Sat" "unsigned" "short" "_Fract"
    (lambda ($4 $3 $2 $1 . $rest)
@@ -455,22 +454,22 @@
      '(fixpt-type "_Sat unsigned long _Accum"))
    ;; struct-or-union-specifier => "struct" opt-attr-specs ident-like "{" s...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (sx-join* 'struct-def $2 $3 (tl->list $5)))
+     (sx-list 'struct-def $2 $3 (tl->list $5)))
    ;; struct-or-union-specifier => "struct" opt-attr-specs "{" struct-decla...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     (sx-join* 'struct-def $2 (tl->list $4)))
+     (sx-list 'struct-def $2 (tl->list $4)))
    ;; struct-or-union-specifier => "struct" opt-attr-specs ident-like
    (lambda ($3 $2 $1 . $rest)
-     (sx-join* 'struct-ref $1 $3))
+     (sx-list 'struct-ref $2 $3))
    ;; struct-or-union-specifier => "union" opt-attr-specs ident-like "{" st...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     (sx-join* 'union-def $2 $3 (tl->list $5)))
+     (sx-list 'union-def $2 $3 (tl->list $5)))
    ;; struct-or-union-specifier => "union" opt-attr-specs "{" struct-declar...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     (sx-join* 'union-def $2 (tl->list $4)))
+     (sx-list 'union-def $2 (tl->list $4)))
    ;; struct-or-union-specifier => "union" opt-attr-specs ident-like
    (lambda ($3 $2 $1 . $rest)
-     (sx-join* 'union-ref $2 $3))
+     (sx-list 'union-ref $2 $3))
    ;; ident-like => identifier
    (lambda ($1 . $rest) $1)
    ;; ident-like => typedef-name
@@ -686,32 +685,32 @@
    ;; direct-declarator => identifier
    (lambda ($1 . $rest) $1)
    ;; direct-declarator => "(" declarator ")"
-   (lambda ($3 $2 $1 . $rest) `(scope ,$2))
+   (lambda ($3 $2 $1 . $rest) $2)
    ;; direct-declarator => "(" attribute-specifier declarator ")"
-   (lambda ($4 $3 $2 $1 . $rest) `(scope ,$2))
+   (lambda ($4 $3 $2 $1 . $rest) $3)
    ;; direct-declarator => direct-declarator "[" type-qualifier-list assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,$3 ,$4))
+     `(ary-declr ,$1 ,$3 ,$4))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list "]"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,$3))
+     `(ary-declr ,$1 ,$3))
    ;; direct-declarator => direct-declarator "[" assignment-expression "]"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,$3))
+     `(ary-declr ,$1 ,$3))
    ;; direct-declarator => direct-declarator "[" "]"
-   (lambda ($3 $2 $1 . $rest) `(array-of ,$1))
+   (lambda ($3 $2 $1 . $rest) `(ary-declr ,$1))
    ;; direct-declarator => direct-declarator "[" "static" type-qualifier-li...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,$4 ,$5))
+     `(ary-declr ,$1 ,$4 ,$5))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list "stati...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,4 ,$5))
+     `(ary-declr ,$1 ,4 ,$5))
    ;; direct-declarator => direct-declarator "[" type-qualifier-list "*" "]"
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(array-of ,$1 ,$3 (var-len)))
+     `(ary-declr ,$1 ,$3 (var-len)))
    ;; direct-declarator => direct-declarator "[" "*" "]"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(array-of ,$1 (var-len)))
+     `(ary-declr ,$1 (var-len)))
    ;; direct-declarator => direct-declarator "(" parameter-type-list ")"
    (lambda ($4 $3 $2 $1 . $rest)
      `(ftn-declr ,$1 ,$3))
@@ -744,7 +743,11 @@
    (lambda ($2 $1 . $rest)
      `(param-decl ,$1 (param-declr ,$2)))
    ;; parameter-declaration => declaration-specifiers
-   (lambda ($1 . $rest) `(param-decl ,$1))
+   (lambda ($1 . $rest)
+     `(param-decl ,$1 (param-declr)))
+   ;; parameter-declaration => declaration-specifiers declarator attribute-...
+   (lambda ($3 $2 $1 . $rest)
+     `(param-decl ,$1 (param-declr ,$2)))
    ;; identifier-list => identifier-list-1
    (lambda ($1 . $rest) (tl->list $1))
    ;; identifier-list-1 => identifier
@@ -756,79 +759,80 @@
    ;; type-name => declaration-specifiers
    (lambda ($1 . $rest) `(type-name ,$1))
    ;; abstract-declarator => pointer direct-abstract-declarator
-   (lambda ($2 $1 . $rest) `(abs-declr ,$1 ,$2))
+   (lambda ($2 $1 . $rest) `(ptr-declr ,$1 ,$2))
    ;; abstract-declarator => pointer
-   (lambda ($1 . $rest) `(abs-declr ,$1))
+   (lambda ($1 . $rest) `(abs-ptr-declr ,$1))
    ;; abstract-declarator => direct-abstract-declarator
-   (lambda ($1 . $rest) `(abs-declr ,$1))
+   (lambda ($1 . $rest) $1)
    ;; direct-abstract-declarator => "(" abstract-declarator ")"
-   (lambda ($3 $2 $1 . $rest) `(declr-scope ,$2))
+   (lambda ($3 $2 $1 . $rest) $2)
+   ;; direct-abstract-declarator => direct-abstract-declarator "(" paramete...
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(ftn-declr ,$1 ,$3))
+   ;; direct-abstract-declarator => direct-abstract-declarator "(" ")"
+   (lambda ($3 $2 $1 . $rest)
+     `(ftn-declr ,$1 (param-list)))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" type-qua...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(declr-array ,$1 ,$3 ,$4))
+     `(ary-declr ,$1 ,$3 ,$4))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" type-qua...
    (lambda ($4 $3 $2 $1 . $rest)
-     `(declr-array ,$1 ,$3))
+     `(ary-declr ,$1 ,$3))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" assignme...
    (lambda ($4 $3 $2 $1 . $rest)
-     `(declr-array ,$1 ,$3))
+     `(ary-declr ,$1 ,$3))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" "]"
-   (lambda ($3 $2 $1 . $rest) `(declr-array ,$1))
+   (lambda ($3 $2 $1 . $rest) `(ary-declr ,$1))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" "static"...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(declr-array
+     `(ary-declr
         ,$1
         ,(tl->list (tl-insert $4 '(stor-spec "static")))
         ,$5))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" "static"...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(declr-array
+     `(ary-declr
         ,$1
         ,(tl->list (tl-insert $4 '(stor-spec "static")))))
    ;; direct-abstract-declarator => direct-abstract-declarator "[" type-qua...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(declr-array
+     `(ary-declr
         ,$1
         ,(tl->list (tl-insert $3 '(stor-spec "static")))
         ,$5))
+   ;; direct-abstract-declarator => direct-abstract-declarator "[" "*" "]"
+   (lambda ($4 $3 $2 $1 . $rest)
+     `(star-ary-declr ,$1))
+   ;; direct-abstract-declarator => "(" parameter-type-list ")"
+   (lambda ($3 $2 $1 . $rest) `(abs-ftn-declr ,$2))
+   ;; direct-abstract-declarator => "(" ")"
+   (lambda ($2 $1 . $rest)
+     '(abs-ftn-declr (param-list)))
    ;; direct-abstract-declarator => "[" type-qualifier-list assignment-expr...
    (lambda ($4 $3 $2 $1 . $rest)
-     `(declr-anon-array ,$2 ,$3))
+     `(abs-ary-declr ,$2 ,$3))
    ;; direct-abstract-declarator => "[" type-qualifier-list "]"
-   (lambda ($3 $2 $1 . $rest)
-     `(declr-anon-array ,$2))
+   (lambda ($3 $2 $1 . $rest) `(abs-ary-declr ,$2))
    ;; direct-abstract-declarator => "[" assignment-expression "]"
-   (lambda ($3 $2 $1 . $rest)
-     `(declr-anon-array ,$2))
+   (lambda ($3 $2 $1 . $rest) `(abs-ary-declr ,$2))
    ;; direct-abstract-declarator => "[" "]"
-   (lambda ($2 $1 . $rest) `(declr-anon-array))
+   (lambda ($2 $1 . $rest) `(abs-ary-declr))
    ;; direct-abstract-declarator => "[" "static" type-qualifier-list assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(declr-anon-array
+     `(abs-ary-declr
         ,(tl->list (tl-insert $3 '(stor-spec "static")))
         ,$4))
    ;; direct-abstract-declarator => "[" "static" type-qualifier-list "]"
    (lambda ($4 $3 $2 $1 . $rest)
-     `(declr-anon-array
+     `(abs-ary-declr
         ,(tl->list (tl-insert $3 '(stor-spec "static")))))
    ;; direct-abstract-declarator => "[" type-qualifier-list "static" assign...
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(declr-anon-array
+     `(abs-ary-declr
         ,(tl->list (tl-insert $2 '(stor-spec "static")))
         ,$4))
-   ;; direct-abstract-declarator => direct-abstract-declarator "[" "*" "]"
-   (lambda ($4 $3 $2 $1 . $rest) `(declr-star ,$1))
    ;; direct-abstract-declarator => "[" "*" "]"
-   (lambda ($3 $2 $1 . $rest) '(declr-star))
-   ;; direct-abstract-declarator => direct-abstract-declarator "(" paramete...
-   (lambda ($4 $3 $2 $1 . $rest)
-     `(abs-ftn-declr ,$1 ,$3))
-   ;; direct-abstract-declarator => direct-abstract-declarator "(" ")"
-   (lambda ($3 $2 $1 . $rest) `(abs-ftn-declr ,$1))
-   ;; direct-abstract-declarator => "(" parameter-type-list ")"
-   (lambda ($3 $2 $1 . $rest) `(anon-ftn-declr ,$2))
-   ;; direct-abstract-declarator => "(" ")"
-   (lambda ($2 $1 . $rest) '(anon-ftn-declr))
+   (lambda ($3 $2 $1 . $rest) '(abs-star-ary-declr))
    ;; typedef-name => 'typename
    (lambda ($1 . $rest) `(typename ,$1))
    ;; initializer => assignment-expression
