@@ -51,13 +51,13 @@
 ;; @end deffn
 (define c99-spec
   (lalr-spec
-   (notice (string-append "Copyright (C) 2016-2018 Matthew R. Wette"
+   (notice (string-append "Copyright (C) 2016-2020 Matthew R. Wette"
 			  license-lgpl3+))
 
    (prec< 'then "else")	       ; "then/else" SR-conflict resolution
-   (prec< 'imp		       ; "implied type" SR-conflict resolution
-	  (nonassoc "char" "short" "int" "long" "_Fract" "_Accum" "_Sat")
-	  (nonassoc "float" "double" "_Complex"))
+   (prec< (left 'imp)	       ; "implied type" SR-conflict resolution
+	  (left "char" "short" "int" "long" "_Fract" "_Accum" "_Sat")
+	  (left "float" "double" "_Complex"))
    (prec< 'shift-on-attr		; living on the edge ...
 	  (nonassoc "__attribute__" "__packed__" "__aligned__" "__alignof__")
 	  'reduce-on-attr
@@ -101,8 +101,8 @@
      (argument-expression-list "," arg-expr-hack ($$ (tl-append $1 $3))))
     (arg-expr-hack
      (declaration-specifiers
-      abstract-declarator ($$ `(param-decl ,(tl->list $1) $2)))
-     (declaration-specifiers ($$ `(param-decl ,(tl->list $1)))))
+      abstract-declarator ($$ `(param-decl ,1 ,$2)))
+     (declaration-specifiers ($$ `(param-decl ,$1))))
 
     (unary-expression
      (postfix-expression)		; S 6.5.3
@@ -308,13 +308,13 @@
      ("long" "_Fract" ($$ '(fixpt-type "long _Fract")))
      ("signed" "short" "_Fract" ($$ '(fixpt-type "signd short _Fract")))
      ("signed" "_Fract" ($$ '(fixpt-type "signed _Fract")))
-     ("signed" "long _Fract" ($$ '(fixpt-type "signed long _Fract")))
+     ("signed" "long" "_Fract" ($$ '(fixpt-type "signed long _Fract")))
      ("unsigned" "short" "_Fract" ($$ '(fixpt-type "unsigned short _Fract")))
      ("unsigned" "_Fract" ($$ '(fixpt-type "unsigned _Fract")))
-     ("unsigned" "long _Fract" ($$ '(fixpt-type "unsigned long _Fract")))
+     ("unsigned" "long" "_Fract" ($$ '(fixpt-type "unsigned long _Fract")))
      ("short" "_Accum" ($$ '(fixpt-type "short _Accum")))
      ("_Accum" ($$ '(fixpt-type "_Accum")))
-     ("long _Accum" ($$ '(fixpt-type "long _Accum")))
+     ("long" "_Accum" ($$ '(fixpt-type "long _Accum")))
      ("signed" "short" "_Accum" ($$ '(fixpt-type "signd short _Accum")))
      ("signed" "_Accum" ($$ '(fixpt-type "signed _Accum")))
      ("signed" "long" "_Accum" ($$ '(fixpt-type "signed long _Accum")))
@@ -327,7 +327,7 @@
      ("_Sat" "signed" "short" "_Fract"
       ($$ '(fixpt-type "_Sat signd short _Fract")))
      ("_Sat" "signed" "_Fract" ($$ '(fixpt-type "_Sat signed _Fract")))
-     ("_Sat" "signed" "long _Fract"
+     ("_Sat" "signed" "long" "_Fract"
       ($$ '(fixpt-type "_Sat signed long _Fract")))
      ("_Sat" "unsigned" "short" "_Fract"
       ($$ '(fixpt-type "_Sat unsigned short _Fract")))
@@ -351,15 +351,17 @@
     ;; This one modified: split out struct-or-union = "struct"|"union"
     (struct-or-union-specifier
      ("struct" opt-attr-specs ident-like "{" struct-declaration-list "}"
-      ($$ (sx-join* 'struct-def $2 $3 (tl->list $5))))
+      ($$ (sx-list 'struct-def $2 $3 (tl->list $5))))
      ("struct" opt-attr-specs "{" struct-declaration-list "}"
-      ($$ (sx-join* 'struct-def $2 (tl->list $4))))
-     ("struct" opt-attr-specs ident-like ($$ (sx-join* 'struct-ref $1 $3)))
+      ($$ (sx-list 'struct-def $2 (tl->list $4))))
+     ("struct" opt-attr-specs ident-like
+      ($$ (sx-list 'struct-ref $2 $3)))
      ("union" opt-attr-specs ident-like "{" struct-declaration-list "}"
-      ($$ (sx-join* 'union-def $2 $3 (tl->list $5))))
+      ($$ (sx-list 'union-def $2 $3 (tl->list $5))))
      ("union" opt-attr-specs "{" struct-declaration-list "}"
-      ($$ (sx-join* 'union-def $2 (tl->list $4))))
-     ("union" opt-attr-specs ident-like ($$ (sx-join* 'union-ref $2 $3))))
+      ($$ (sx-list 'union-def $2 (tl->list $4))))
+     ("union" opt-attr-specs ident-like
+      ($$ (sx-list 'union-ref $2 $3))))
 
     ;; because name following struct/union can be identifier or typeref:
     (ident-like
@@ -374,8 +376,10 @@
     (struct-declaration-list		; S 6.7.2.1
      (struct-declaration ($$ (make-tl 'field-list $1)))
      (lone-comment ($$ (make-tl 'field-list $1)))
+     ;;(cpp-statement ($$ (make-tl 'field-list $1)))
      (struct-declaration-list struct-declaration ($$ (tl-append $1 $2)))
      (struct-declaration-list lone-comment ($$ (tl-append $1 $2)))
+     ;;(struct-declaration-list cpp-statement ($$ (tl-append $1 $2)))
      ;; Not in C99, but allowed by GNU, I believe:
      (";" ($$ (make-tl 'field-list)))
      (struct-declaration-list ";" ($$ $1)))
@@ -510,6 +514,7 @@
      (init-declarator-list-1 "," attribute-specifiers
 			     init-declarator ($$ (tl-append $1 $3 $4))))
 
+    ;; todo: move asm-expression to attribute
     (init-declarator			; S 6.7
      (init-declarator-1 ($$ (process-declr $1))))
     (init-declarator-1
@@ -538,29 +543,30 @@
     (direct-declarator			; S 6.7.6
      (identifier ($$ $1))
      ;;(ident-like ($$ $1))
-     ("(" declarator ")" ($$ `(scope ,$2)))
-     ("(" attribute-specifier declarator ")" ($$ `(scope ,$2)))
+     ;;("(" declarator ")" ($$ `(scope ,$2)))
+     ("(" declarator ")" ($$ $2))
+     ("(" attribute-specifier declarator ")" ($$ $3))
      (direct-declarator
       "[" type-qualifier-list assignment-expression "]"
-      ($$ `(array-of ,$1 ,$3 ,$4)))
+      ($$ `(ary-declr ,$1 ,$3 ,$4)))
      (direct-declarator
-      "[" type-qualifier-list "]" ($$ `(array-of ,$1 ,$3)))
+      "[" type-qualifier-list "]" ($$ `(ary-declr ,$1 ,$3)))
      (direct-declarator
-      "[" assignment-expression "]" ($$ `(array-of ,$1 ,$3)))
+      "[" assignment-expression "]" ($$ `(ary-declr ,$1 ,$3)))
      (direct-declarator
-      "[" "]" ($$ `(array-of ,$1)))
+      "[" "]" ($$ `(ary-declr ,$1)))
      (direct-declarator
       "[" "static" type-qualifier-list assignment-expression "]"
-      ($$ `(array-of ,$1 ,$4 ,$5))) ;; FIXME $4 needs "static" added
+      ($$ `(ary-declr ,$1 ,$4 ,$5))) ;; FIXME $4 needs "static" added
      (direct-declarator
       "[" type-qualifier-list "static" assignment-expression "]"
-      ($$ `(array-of ,$1 ,4 ,$5))) ;; FIXME $4 needs "static" added
+      ($$ `(ary-declr ,$1 ,4 ,$5))) ;; FIXME $4 needs "static" added
      (direct-declarator
       "[" type-qualifier-list "*" "]"	; variable length array
-      ($$ `(array-of ,$1 ,$3 (var-len))))
+      ($$ `(ary-declr ,$1 ,$3 (var-len))))
      (direct-declarator
       "[" "*" "]"			; variable length array
-      ($$ `(array-of ,$1 (var-len))))
+      ($$ `(ary-declr ,$1 (var-len))))
      (direct-declarator
       "(" parameter-type-list ")" ($$ `(ftn-declr ,$1 ,$3)))
      (direct-declarator
@@ -588,7 +594,10 @@
      (declaration-specifiers
       abstract-declarator ($$ `(param-decl ,$1 (param-declr ,$2))))
      (declaration-specifiers
-      ($$ `(param-decl ,$1))))
+      ($$ `(param-decl ,$1 (param-declr))))
+     ;; adding attribute specifiers:
+     (declaration-specifiers
+      declarator attribute-specifiers ($$ `(param-decl ,$1 (param-declr ,$2)))))
 
     (identifier-list
      (identifier-list-1 ($$ (tl->list $1))))
@@ -604,55 +613,52 @@
      (declaration-specifiers ($$ `(type-name ,$1))))
 
     (abstract-declarator		; S 6.7.6
-     (pointer direct-abstract-declarator ($$ `(abs-declr ,$1 ,$2)))
-     (pointer ($$ `(abs-declr ,$1)))
-     (direct-abstract-declarator ($$ `(abs-declr ,$1))))
+     (pointer direct-abstract-declarator ($$ `(ptr-declr ,$1 ,$2)))
+     (pointer ($$ `(abs-ptr-declr ,$1)))
+     (direct-abstract-declarator))
 
     (direct-abstract-declarator
-     ("(" abstract-declarator ")" ($$ `(declr-scope ,$2)))
+     ("(" abstract-declarator ")" ($$ $2))
+     (direct-abstract-declarator "(" parameter-type-list ")"
+				 ($$ `(ftn-declr ,$1 ,$3)))
+     (direct-abstract-declarator "(" ")" ($$ `(ftn-declr ,$1 (param-list))))
      (direct-abstract-declarator
       "[" type-qualifier-list assignment-expression "]"
-      ($$ `(declr-array ,$1 ,$3 ,$4)))
+      ($$ `(ary-declr ,$1 ,$3 ,$4)))
      (direct-abstract-declarator
       "[" type-qualifier-list "]"
-      ($$ `(declr-array ,$1 ,$3)))
+      ($$ `(ary-declr ,$1 ,$3)))
      (direct-abstract-declarator
       "[" assignment-expression "]"
-      ($$ `(declr-array ,$1 ,$3)))
+      ($$ `(ary-declr ,$1 ,$3)))
      (direct-abstract-declarator
-      "[" "]" ($$ `(declr-array ,$1)))
+      "[" "]" ($$ `(ary-declr ,$1)))
      (direct-abstract-declarator
       "[" "static" type-qualifier-list assignment-expression "]"
-      ($$ `(declr-array
-	    ,$1 ,(tl->list (tl-insert $4 '(stor-spec "static"))) ,$5)))
+      ($$ `(ary-declr ,$1 ,(tl->list (tl-insert $4 '(stor-spec "static"))) ,$5)))
      (direct-abstract-declarator
       "[" "static" type-qualifier-list "]"
-      ($$ `(declr-array ,$1 ,(tl->list (tl-insert $4 '(stor-spec "static"))))))
+      ($$ `(ary-declr ,$1 ,(tl->list (tl-insert $4 '(stor-spec "static"))))))
      (direct-abstract-declarator
       "[" type-qualifier-list "static" assignment-expression "]"
-      ($$ `(declr-array
-	    ,$1 ,(tl->list (tl-insert $3 '(stor-spec "static"))) ,$5)))
+      ($$ `(ary-declr ,$1 ,(tl->list (tl-insert $3 '(stor-spec "static"))) ,$5)))
+     (direct-abstract-declarator "[" "*" "]" ($$ `(star-ary-declr ,$1)))
      ;;
+     ("(" parameter-type-list ")" ($$ `(abs-ftn-declr ,$2)))
+     ("(" ")" ($$ '(abs-ftn-declr (param-list))))
      ("[" type-qualifier-list assignment-expression "]"
-      ($$ `(declr-anon-array ,$2 ,$3)))
-     ("[" type-qualifier-list "]" ($$ `(declr-anon-array ,$2)))
-     ("[" assignment-expression "]" ($$ `(declr-anon-array ,$2)))
-     ("[" "]" ($$ `(declr-anon-array)))
+      ($$ `(abs-ary-declr ,$2 ,$3)))
+     ("[" type-qualifier-list "]" ($$ `(abs-ary-declr ,$2)))
+     ("[" assignment-expression "]" ($$ `(abs-ary-declr ,$2)))
+     ("[" "]" ($$ `(abs-ary-declr)))
      ("[" "static" type-qualifier-list assignment-expression "]"
-      ($$ `(declr-anon-array
-	    ,(tl->list (tl-insert $3 '(stor-spec "static"))) ,$4)))
+      ($$ `(abs-ary-declr ,(tl->list (tl-insert $3 '(stor-spec "static"))) ,$4)))
      ("[" "static" type-qualifier-list "]"
-      ($$ `(declr-anon-array ,(tl->list (tl-insert $3 '(stor-spec "static"))))))
+      ($$ `(abs-ary-declr ,(tl->list (tl-insert $3 '(stor-spec "static"))))))
      ("[" type-qualifier-list "static" assignment-expression "]"
-      ($$ `(declr-anon-array
-	    ,(tl->list (tl-insert $2 '(stor-spec "static"))) ,$4)))
-     (direct-abstract-declarator "[" "*" "]" ($$ `(declr-star ,$1)))
-     ("[" "*" "]" ($$ '(declr-star)))
-     (direct-abstract-declarator "(" parameter-type-list ")"
-				 ($$ `(abs-ftn-declr ,$1 ,$3)))
-     (direct-abstract-declarator "(" ")" ($$ `(abs-ftn-declr ,$1)))
-     ("(" parameter-type-list ")" ($$ `(anon-ftn-declr ,$2)))
-     ("(" ")" ($$ '(anon-ftn-declr))))
+      ($$ `(abs-ary-declr ,(tl->list (tl-insert $2 '(stor-spec "static"))) ,$4)))
+     ("[" "*" "]" ($$ '(abs-star-ary-declr)))
+     )
 
     ;; typedef-name is generated by the lexical analyzer
     (typedef-name ('typename ($$ `(typename ,$1))))
@@ -681,7 +687,6 @@
     (designator
      ("[" constant-expression "]" ($$ `(array-dsgr ,$2)))
      ("." identifier ($$ `(sel-dsgr ,$2))))
-
     ;; === statements =========================================================
 
     (statement
@@ -859,7 +864,7 @@
    (hashify-machine
     (make-lalr-machine c99-spec))
    #:keep 2
-   #:keepers '($code-comm $lone-comm $pragma)))
+   #:keepers '($code-comm $lone-comm $pragma cpp-stmt)))
 
 (define c99x-spec (restart-spec c99-spec 'expression))
 
@@ -868,7 +873,8 @@
    (hashify-machine
     (make-lalr-machine c99x-spec))
    #:keep 2
-   #:keepers '($code-comm $lone-comm $pragma)))
+   ;; Sync w/ #:skip-if-unexp arg to make-lalr-parser in parser.scm.
+   #:keepers '($code-comm $lone-comm $pragma cpp-stmt)))
 
 ;;; =====================================
 
