@@ -44,10 +44,11 @@
 			      (if (> i n) r (loop (cons (mkarg i) r) (1+ i)))))
   (cons* 'lambda (make-arg-list (car actn)) (cdr actn)))
 
-(define (make-xct av)
+(define (make-xct av env)
   (if (procedure? (vector-ref av 0))
       av
-      (vector-map (lambda (ix f) (eval f (current-module)))
+      (vector-map (lambda (ix f)
+		    (eval f (or env (current-module))))
 		  (vector-map (lambda (ix actn) (wrap-action actn)) av))))
 
 (define (sferr fmt . args)
@@ -77,11 +78,11 @@
 	   "~A:~A: parse failed at state ~A, on input ~S"
 	   fn ln (car state) (cdr laval))))
 
-(define* (make-lalr-parser/sym mach #:key (skip-if-unexp '()) interactive)
+(define* (make-lalr-parser/sym mach #:key (skip-if-unexp '()) interactive env)
   (let* ((len-v (assq-ref mach 'len-v))
 	 (rto-v (assq-ref mach 'rto-v))
 	 (pat-v (assq-ref mach 'pat-v))
-	 (xct-v (make-xct (assq-ref mach 'act-v)))
+	 (xct-v (make-xct (assq-ref mach 'act-v) env))
 	 (start (assq-ref (assq-ref mach 'mtab) '$start)))
     (lambda* (lexr #:key debug)
       (let loop ((state (list 0))	; state stack
@@ -102,7 +103,9 @@
 		 (tval (car laval))
 		 (sval (cdr laval))
 		 (stxl (vector-ref pat-v (car state)))
-		 (stx (or (assq-ref stxl tval) (assq-ref stxl '$default)
+		 (stx (or (assq-ref stxl tval)
+			  (and (not (memq tval skip-if-unexp))
+			       (assq-ref stxl '$default))
 			  (cons 'error #f))))
 	    (if debug (dmsg/s (car state) (if nval tval sval) stx))
 	    (cond
@@ -124,11 +127,11 @@
 	     (else			; accept
 	      (car stack))))))))))
 
-(define* (make-lalr-parser/num mach #:key (skip-if-unexp '()) interactive)
+(define* (make-lalr-parser/num mach #:key (skip-if-unexp '()) interactive env)
   (let* ((len-v (assq-ref mach 'len-v))
 	 (rto-v (assq-ref mach 'rto-v))
 	 (pat-v (assq-ref mach 'pat-v))
-	 (xct-v (make-xct (assq-ref mach 'act-v)))
+	 (xct-v (make-xct (assq-ref mach 'act-v) env))
 	 (ntab (assq-ref mach 'ntab))
 	 (start (assq-ref (assq-ref mach 'mtab) '$start)))
     (lambda* (lexr #:key debug)
@@ -152,7 +155,8 @@
 		 (stxl (vector-ref pat-v (car state)))
 		 (stx (or (assq-ref stxl tval)
 			  (and (not (memq tval skip-if-unexp))
-			       (assq-ref stxl $default)))))
+			       (assq-ref stxl $default))
+			  #f)))		; error
 	    (if debug (dmsg/n (car state) (if nval tval sval) stx ntab))
 	    (cond
 	     ((eq? #f stx)		; error
@@ -196,10 +200,16 @@
 ;; If @code{#t}, this tells the parser that this is being called
 ;; interactively, so that the token @code{$end} is not expected.
 ;; The default value is @code{#f}.
+;; @item #:env
+;; Use the passed environment to the parser actions provided in the
+;; specification.  This can be a module (e.g., @code{(current-module)}
+;; or an environment (e.g., @code{(interaction-environment)}).   The
+;; default is to use @code{(current-module)}; that is, the module in which
+;; this procedure is being executed.
 ;; @end table
 ;; @noindent
 ;; @end deffn
-(define* (make-lalr-parser mach #:key (skip-if-unexp '()) interactive)
+(define* (make-lalr-parser mach #:key (skip-if-unexp '()) interactive env)
   "- Procedure: make-lalr-parser mach [options] => parser
      Generate a procedure for parsing a language, where MACH is a
      machine generated by 'make-lalr-machine'.  This generates a
@@ -216,14 +226,27 @@
      '#:interactive'
           If '#t', this tells the parser that this is being called
           interactively, so that the token '$end' is not expected.  The
-          default value is '#f'."
+          default value is '#f'.
+     '#:env'
+          Use the passed environment to the parser actions provided in
+          the specification.  This can be a module (e.g.,
+          '(current-module)' or an environment (e.g.,
+          '(interaction-environment)').  The default is to use
+          '(current-module)'; that is, the module in which this
+          procedure is being executed."
   (let* ((mtab (assq-ref mach 'mtab))
 	 (siu (map (lambda (n) (assoc-ref mtab n)) skip-if-unexp))
 	 (iact interactive))
     (if (number? (caar (vector-ref (assq-ref mach 'pat-v) 0)))
 	;; hashed:
-	(make-lalr-parser/num mach #:skip-if-unexp siu #:interactive iact)
+	(make-lalr-parser/num mach
+			      #:skip-if-unexp siu
+			      #:interactive iact
+			      #:env env)
 	;; not hashed:
-	(make-lalr-parser/sym mach #:skip-if-unexp siu #:interactive iact))))
+	(make-lalr-parser/sym mach
+			      #:skip-if-unexp siu
+			      #:interactive iact
+			      #:env env))))
 
 ;;; --- last line ---
