@@ -27,14 +27,14 @@
   #:use-module (nyacc lang c99 ffi-help)
   #:use-module (system base language)
   #:use-module ((system base compile) #:select (compile-file))
-  #:use-module (srfi srfi-1)
+  #:use-module ((srfi srfi-1) #:select (fold fold-right))
   #:use-module (srfi srfi-37)
-  #:version (1 02 2))
+  #:version (1 03 0))
 
-(define *ffi-help-version* "1.02.2")
+(define *ffi-help-version* "1.03.0")
 
 (define %summary
-  "Compile a ffi-file (C interface spec) to Scheme (or maybe .go).")
+  "Compile a ffi-file to .scm and maybe .go.")
 
 (define (sfmt fmt . args)
   (apply simple-format (current-error-port) fmt args)
